/*
 * Copyright 2017, Inventek Systems. All Rights Reserved. This software, associated documentation
 * and materials ("Software"), referenced and provided with this documentation is owned by Inventek
 * Systems and is protected by and subject to worldwide patent protection (United States and foreign),
 * United States copyright laws and international treaty provisions. Therefore, you may use this
 * Software only as provided in the license agreement accompanying the software package from which
 * you obtained this Software ("EULA"). If no EULA applies, Inventek Systems hereby grants you a
 * personal, non-exclusive, non-transferable license to copy, modify, and compile the Software source
 * code solely for use in connection with Inventek's integrated circuit products.
 *
 * Any reproduction, modification, translation, compilation, or representation of this Software except
 * as specified above is prohibited without the express written permission of Inventek.
 * Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE.
 * 
 * Inventek reserves the right to make changes to the Software without notice. Inventek does not
 * assume any liability arising out of the application or use of the Software or any product or
 * circuit described in the Software. Inventek does not authorize its products for use in any
 * products where a malfunction or failure of the Inventek product may reasonably be expected to
 * result in significant property damage, injury, or death ("High Risk Product"). By including
 * Inventek's product in a High Risk product, the manufacturer of such system or application
 * assumes all risk of such use and in doing so agrees to indemnify Inventek against all liability.
 * Inventek Systems reserves the right to make changes without further notice to any products or
 * data herein to improve reliability, function, or design. The information contained within is
 * believed to be accurate and reliable. However Inventek does not assume any liability arising
 * out of the application or use of this information, nor the application or use of any product
 * or circuit described herein, neither does it convey any license under its patent rights nor
 * the rights of others.
 */

/** @file
 * Defines board support package for ISM943903_R48_L54 board
 */
#include "stdio.h"
#include "string.h"
#include "gpio_irq.h"
#include "platform_config.h"
#include "platform_peripheral.h"
#include "platform_mcu_peripheral.h"
#include "platform_ethernet.h"
#include "wwd_assert.h"
#include "platform/wwd_platform_interface.h"
#include "wiced_platform.h"
#include "platform_appscr4.h"
#include "platform_bluetooth.h"
#include "platform_mfi.h"
#include "gpio_button.h"
#include "./platform.h"

#include "platform_button.h"
#include "gpio_button.h"
/* **** AT Commands Only **** */
#ifndef WICED_APP
#endif
/* **** AT Commands Only **** */

/******************************************************
 *                      Macros
 ******************************************************/

/******************************************************
 *                    Constants
 ******************************************************/

/******************************************************
 *                   Enumerations
 ******************************************************/

/******************************************************
 *                 Type Definitions
 ******************************************************/

/******************************************************
 *                    Structures
 ******************************************************/

/******************************************************
 *               Variables Definitions
 ******************************************************/
extern const platform_gpio_t platform_gpio_pins[];

/* Platform pin mapping table. Used by WICED/platform/MCU/wiced_platform_common.c.*/
const platform_gpio_t platform_gpio_pins[] =
{
    //                                                  eS-WiFi Signal          L44         +10/L54
    /* JTAG */
    [WICED_GPIO_18         ]    = { PIN_GPIO_3 },       //TMS                    4
    [WICED_GPIO_17         ]    = { PIN_GPIO_2 },       //TCK                    5
    [WICED_GPIO_19         ]    = { PIN_GPIO_4 },       //TDI                    6
    [WICED_GPIO_20         ]    = { PIN_GPIO_5 },       //TDO                    7
    [WICED_GPIO_21         ]    = { PIN_GPIO_6 },       //TRSTN                  8

    /* SPI */
    [WICED_GPIO_15         ]    = { PIN_SPI_0_MOSI },   //SPI_MOSI               9
    [WICED_GPIO_13         ]    = { PIN_SPI_0_MISO },   //SPI_MISO              10
    [WICED_GPIO_14         ]    = { PIN_SPI_0_CLK },    //SPI_CLK               11
    [WICED_GPIO_16         ]    = { PIN_SPI_0_CS },     //SPI_SSN               12
    [WICED_GPIO_3          ]    = { PIN_GPIO_7 },       //DATARDY/I2C1_CLK      13

    /* COMM */
    [WICED_PERIPHERAL_PIN_6]    = { PIN_UART0_RTS },    //RTS                   18
    [WICED_PERIPHERAL_PIN_5]    = { PIN_UART0_CTS },    //CTS                   19
    [WICED_PERIPHERAL_PIN_3]    = { PIN_UART0_RXD },    //RX                    21
    [WICED_PERIPHERAL_PIN_4]    = { PIN_UART0_TXD },    //TX                    22

    /* GPIO */
    [WICED_GPIO_1          ]    = { PIN_GPIO_0 },       //GPIO0/I2C1_SD         23
    [WICED_GPIO_2          ]    = { PIN_GPIO_1 },       //GPIO1                 24
    [WICED_GPIO_4          ]    = { PIN_GPIO_8 },       //GPIO3                 26
    [WICED_GPIO_5          ]    = { PIN_GPIO_9 },       //GPIO2                 25
    [WICED_GPIO_6          ]    = { PIN_GPIO_10 },      //GPIO4                 27

    /* CFG */
    [WICED_PERIPHERAL_PIN_2]    = { PIN_RF_SW_CTRL_7 }, //CFG0                  28
    [WICED_PERIPHERAL_PIN_1]    = { PIN_RF_SW_CTRL_6 }, //CFG1                  29

    /* RES GPIO */
    [WICED_GPIO_10         ]    = { PIN_GPIO_14 },      //GPIO15                30
    [WICED_GPIO_8          ]    = { PIN_GPIO_12 },      //GPIO14                31
    [WICED_GPIO_9          ]    = { PIN_GPIO_13 },      //GPIO13                32

    /* +10 */
    [WICED_GPIO_12         ]    = { PIN_GPIO_16 },      //GPIOA                             45
    [WICED_GPIO_7          ]    = { PIN_GPIO_11 },      //GPIOB                             46
                                                        //GPIOC                             47 (NC)
    [WICED_PERIPHERAL_PIN_7]    = { PIN_RF_SW_CTRL_8 }, //GPIOD                             48
    [WICED_PERIPHERAL_PIN_8]    = { PIN_RF_SW_CTRL_9 }, //GPIOE                             49
                                                        //GPIOF                             50 (NC)
                                                        //GPIOG                             51 (NC)
    [WICED_GPIO_23         ]    = { PIN_I2C0_CLK },     //GPIOH                             52
    [WICED_GPIO_22         ]    = { PIN_I2C0_SDATA },   //GPIOI                             53

    /* MISC */
    [WICED_GPIO_11         ]    = { PIN_GPIO_15 },      //NA

};

const platform_spi_t platform_spi_peripherals[] =
{
    [WICED_SPI_1]  =
    {
        .port                    = BCM4390X_SPI_0,
        .pin_mosi                = &platform_gpio_pins[WICED_GPIO_15],
        .pin_miso                = &platform_gpio_pins[WICED_GPIO_13],
        .pin_clock               = &platform_gpio_pins[WICED_GPIO_14],
        .pin_cs                  = &platform_gpio_pins[WICED_GPIO_16],
        .driver                  = &spi_gsio_driver,
    },
};


/* I2C peripherals. Used by WICED/platform/MCU/wiced_platform_common.c */
const platform_i2c_t platform_i2c_peripherals[] =
{
    [WICED_I2C_1] =
    {
        .port                    = BCM4390X_I2C_1,
        .pin_sda                 = &platform_gpio_pins[WICED_GPIO_1],
        .pin_scl                 = &platform_gpio_pins[WICED_GPIO_3],
        .driver                  = &i2c_gsio_driver,
    },
};

/* UART peripherals and runtime drivers. Used by WICED/platform/MCU/wiced_platform_common.c */
const platform_uart_t platform_uart_peripherals[] =
{
    [WICED_UART_1] = /* ChipCommon Slow UART */
    {
        .port    = UART_SLOW,
        .rx_pin  = &platform_gpio_pins[WICED_PERIPHERAL_PIN_1],
        .tx_pin  = &platform_gpio_pins[WICED_PERIPHERAL_PIN_2],
        .cts_pin = NULL,
        .rts_pin = NULL,
        .src_clk = CLOCK_ALP,
    },
    [WICED_UART_2] = /* ChipCommon Fast UART */
    {
        .port    = UART_FAST,
        .rx_pin  = &platform_gpio_pins[WICED_PERIPHERAL_PIN_3],
        .tx_pin  = &platform_gpio_pins[WICED_PERIPHERAL_PIN_4],
        .cts_pin = &platform_gpio_pins[WICED_PERIPHERAL_PIN_5],
        .rts_pin = &platform_gpio_pins[WICED_PERIPHERAL_PIN_6],
        .src_clk = CLOCK_HT,
    },
    [WICED_UART_3] = /* GCI UART */
    {
        .port    = UART_GCI,
        .rx_pin  = &platform_gpio_pins[WICED_PERIPHERAL_PIN_7],
        .tx_pin  = &platform_gpio_pins[WICED_PERIPHERAL_PIN_8],
        .cts_pin = NULL,
        .rts_pin = NULL,
        .src_clk = CLOCK_ALP,
    }
};
platform_uart_driver_t platform_uart_drivers[WICED_UART_MAX];

#ifdef DEFAULT_UART_BAUD
#pragma message "Custom UART Baud rate"
#endif

/* UART standard I/O configuration */
#ifndef WICED_DISABLE_STDIO
static const platform_uart_config_t stdio_config =
{
    .baud_rate    = 115200,
    .data_width   = DATA_WIDTH_8BIT,
    .parity       = NO_PARITY,
    .stop_bits    = STOP_BITS_1,
    .flow_control = FLOW_CONTROL_DISABLED,
};
#endif

const platform_hibernation_t hibernation_config =
{
    .clock              = PLATFORM_HIBERNATION_CLOCK_EXTERNAL,
    .hib_ext_clock_freq = 0, /* use default settings */
    .rc_code            = PLATFORM_HIB_WAKE_CTRL_REG_RCCODE,
};

gpio_button_t platform_gpio_buttons[] =
{
    [PLATFORM_BUTTON_1] =
    {
        .polarity   = WICED_ACTIVE_HIGH,
        .gpio       = WICED_BUTTON1,
        .trigger    = IRQ_TRIGGER_BOTH_EDGES,
    },

    [PLATFORM_BUTTON_2] =
    {
        .polarity   = WICED_ACTIVE_HIGH,
        .gpio       = WICED_BUTTON2,
        .trigger    = IRQ_TRIGGER_BOTH_EDGES,
    },
};

#ifdef MFI
/* MFI-related variables */
const wiced_i2c_device_t auth_chip_i2c_device =
{
    .port          = AUTH_IC_I2C_PORT,
    .address       = 0x11,
    .address_width = I2C_ADDRESS_WIDTH_7BIT,
    .speed_mode    = I2C_STANDARD_SPEED_MODE,
};

const platform_mfi_auth_chip_t platform_auth_chip =
{
    .i2c_device = &auth_chip_i2c_device,
    .reset_pin  = WICED_GPIO_AUTH_RST
};
#endif

const wiced_gpio_t platform_gpio_leds[PLATFORM_LED_COUNT] =
{
     [WICED_LED_INDEX_1] = WICED_LED1,
     [WICED_LED_INDEX_2] = WICED_LED2,
};

/******************************************************
 *               Function Declarations
 ******************************************************/

/******************************************************
 *               Function Definitions
 ******************************************************/

platform_result_t platform_gpio_set_state(int gpio_index, int off_on )
{
    switch (off_on)
    {
        case WICED_TRUE:
            platform_gpio_output_low( &platform_gpio_pins[gpio_index] );
            break;
        case WICED_FALSE:
            platform_gpio_output_high( &platform_gpio_pins[gpio_index] );
            break;
    }
    return PLATFORM_SUCCESS;
}


/* LEDs on this platform are active HIGH */
platform_result_t platform_led_set_state(int led_index, int off_on )
{
    if ((led_index >= 0) && (led_index < PLATFORM_LED_COUNT))
    {
        switch (off_on)
        {
            case WICED_LED_OFF:
                platform_gpio_output_low( &platform_gpio_pins[platform_gpio_leds[led_index]] );
                break;
            case WICED_LED_ON:
                platform_gpio_output_high( &platform_gpio_pins[platform_gpio_leds[led_index]] );
                break;
        }
        return PLATFORM_SUCCESS;
    }
    return PLATFORM_BADARG;
}

void platform_led_init( void )
{
    /* Initialise LEDs and turn off by default */
    platform_gpio_init( &platform_gpio_pins[WICED_LED1], OUTPUT_PUSH_PULL );
    platform_gpio_init( &platform_gpio_pins[WICED_LED2], OUTPUT_PUSH_PULL );
    platform_led_set_state(WICED_LED_INDEX_1, WICED_LED_ON);
    platform_led_set_state(WICED_LED_INDEX_2, WICED_LED_OFF);
 }


void platform_init_external_devices( void )
{
    /* Initialise LEDs and turn off by default */
    platform_led_init();


    /* Initialise buttons to input by default */
    platform_gpio_init( &platform_gpio_pins[WICED_BUTTON1], INPUT_PULL_UP );
    platform_gpio_init( &platform_gpio_pins[WICED_BUTTON2], INPUT_PULL_UP );

#ifndef WICED_DISABLE_STDIO
    /* Initialise UART standard I/O */
    platform_stdio_init( &platform_uart_drivers[STDIO_UART], &platform_uart_peripherals[STDIO_UART], &stdio_config );
#endif
}
